@@ -1,4 +1,4 @@
-/*      $Id: display.c 28365 2008-10-22 20:10:00Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -113,6 +113,7 @@ myDisplayInitAtoms (DisplayInfo *display_info)
         "_NET_WM_ALLOWED_ACTIONS",
         "_NET_WM_CONTEXT_HELP",
         "_NET_WM_DESKTOP",
+        "_NET_WM_FULLSCREEN_MONITORS",
         "_NET_WM_ICON",
         "_NET_WM_ICON_GEOMETRY",
         "_NET_WM_ICON_NAME",
@@ -338,7 +339,8 @@ myDisplayInit (GdkDisplay *gdisplay)
     display->screens = NULL;
     display->clients = NULL;
     display->xgrabcount = 0;
-    display->dbl_click_time = 300;
+    display->double_click_time = 250;
+    display->double_click_distance = 5;
     display->nb_screens = 0;
     display->current_time = CurrentTime;
 
@@ -397,6 +399,14 @@ myDisplayHaveShape (DisplayInfo *display)
     return (display->have_shape);
 }
 
+gboolean
+myDisplayHaveShapeInput (DisplayInfo *display)
+{
+    g_return_val_if_fail (display != NULL, FALSE);
+
+    return ((display->have_shape) && (display->shape_version >= 1001));
+}
+
 gboolean
 myDisplayHaveRender (DisplayInfo *display)
 {
@@ -430,12 +440,12 @@ myDisplayGetCursorRoot (DisplayInfo *display)
 }
 
 Cursor
-myDisplayGetCursorResize (DisplayInfo *display, guint index)
+myDisplayGetCursorResize (DisplayInfo *display, guint list)
 {
     g_return_val_if_fail (display, None);
-    g_return_val_if_fail (index < 8, None);
+    g_return_val_if_fail (list < 8, None);
 
-    return display->resize_cursor [index];
+    return display->resize_cursor [list];
 }
 
 
@@ -495,14 +505,14 @@ myDisplayRemoveClient (DisplayInfo *display, Client *c)
 Client *
 myDisplayGetClientFromWindow (DisplayInfo *display, Window w, unsigned short mode)
 {
-    GSList *index;
+    GSList *list;
 
     g_return_val_if_fail (w != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
 
-    for (index = display->clients; index; index = g_slist_next (index))
+    for (list = display->clients; list; list = g_slist_next (list))
     {
-        Client *c = (Client *) index->data;
+        Client *c = (Client *) list->data;
         if (clientGetFromWindow (c, w, mode))
         {
             return (c);
@@ -540,14 +550,14 @@ myDisplayRemoveScreen (DisplayInfo *display, ScreenInfo *screen)
 ScreenInfo *
 myDisplayGetScreenFromRoot (DisplayInfo *display, Window root)
 {
-    GSList *index;
+    GSList *list;
 
     g_return_val_if_fail (root != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
 
-    for (index = display->screens; index; index = g_slist_next (index))
+    for (list = display->screens; list; list = g_slist_next (list))
     {
-        ScreenInfo *screen = (ScreenInfo *) index->data;
+        ScreenInfo *screen = (ScreenInfo *) list->data;
         if (screen->xroot == root)
         {
             return screen;
@@ -561,13 +571,13 @@ myDisplayGetScreenFromRoot (DisplayInfo *display, Window root)
 ScreenInfo *
 myDisplayGetScreenFromNum (DisplayInfo *display, int num)
 {
-    GSList *index;
+    GSList *list;
 
     g_return_val_if_fail (display != NULL, NULL);
 
-    for (index = display->screens; index; index = g_slist_next (index))
+    for (list = display->screens; list; list = g_slist_next (list))
     {
-        ScreenInfo *screen = (ScreenInfo *) index->data;
+        ScreenInfo *screen = (ScreenInfo *) list->data;
         if (screen->screen == num)
         {
             return screen;
@@ -616,14 +626,14 @@ myDisplayGetScreenFromWindow (DisplayInfo *display, Window w)
 ScreenInfo *
 myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 {
-    GSList *index;
+    GSList *list;
 
     g_return_val_if_fail (w != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
 
-    for (index = display->screens; index; index = g_slist_next (index))
+    for (list = display->screens; list; list = g_slist_next (list))
     {
-        ScreenInfo *screen = (ScreenInfo *) index->data;
+        ScreenInfo *screen = (ScreenInfo *) list->data;
         if (screen->systray == w)
         {
             return screen;
@@ -637,17 +647,17 @@ myDisplayGetScreenFromSystray (DisplayInfo *display, Window w)
 
 #ifdef HAVE_XSYNC
 Client *
-myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm alarm)
+myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm xalarm)
 {
-    GSList *index;
+    GSList *list;
 
-    g_return_val_if_fail (alarm != None, NULL);
+    g_return_val_if_fail (xalarm != None, NULL);
     g_return_val_if_fail (display != NULL, NULL);
 
-    for (index = display->clients; index; index = g_slist_next (index))
+    for (list = display->clients; list; list = g_slist_next (list))
     {
-        Client *c = (Client *) index->data;
-        if (alarm == c->xsync_alarm)
+        Client *c = (Client *) list->data;
+        if (xalarm == c->xsync_alarm)
         {
             return (c);
         }
@@ -661,14 +671,14 @@ myDisplayGetClientFromXSyncAlarm (DisplayInfo *display, XSyncAlarm alarm)
 ScreenInfo *
 myDisplayGetDefaultScreen (DisplayInfo *display)
 {
-    GSList *index;
+    GSList *list;
 
     g_return_val_if_fail (display != NULL, NULL);
 
-    index = display->screens;
-    if (index)
+    list = display->screens;
+    if (list)
     {
-        return (ScreenInfo *) index->data;
+        return (ScreenInfo *) list->data;
     }
 
     return NULL;
@@ -677,48 +687,62 @@ myDisplayGetDefaultScreen (DisplayInfo *display)
 guint32
 myDisplayUpdateCurrentTime (DisplayInfo *display, XEvent *ev)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    guint32 timestamp;
 
+    g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
+
+    timestamp = (guint32) CurrentTime;
     switch (ev->type)
     {
         case KeyPress:
         case KeyRelease:
-            display->current_time = (guint32) ev->xkey.time;
+            timestamp = (guint32) ev->xkey.time;
             break;
         case ButtonPress:
         case ButtonRelease:
-            display->current_time = (guint32) ev->xbutton.time;
+            timestamp = (guint32) ev->xbutton.time;
             break;
         case MotionNotify:
-            display->current_time = (guint32) ev->xmotion.time;
+            timestamp = (guint32) ev->xmotion.time;
             break;
         case EnterNotify:
         case LeaveNotify:
-            display->current_time = (guint32) ev->xcrossing.time;
+            timestamp = (guint32) ev->xcrossing.time;
             break;
         case PropertyNotify:
-            display->current_time = (guint32) ev->xproperty.time;
+            timestamp = (guint32) ev->xproperty.time;
             break;
         case SelectionClear:
-            display->current_time = (guint32) ev->xselectionclear.time;
+            timestamp = (guint32) ev->xselectionclear.time;
             break;
         case SelectionRequest:
-            display->current_time = (guint32) ev->xselectionrequest.time;
+            timestamp = (guint32) ev->xselectionrequest.time;
             break;
         case SelectionNotify:
-            display->current_time = (guint32) ev->xselection.time;
+            timestamp = (guint32) ev->xselection.time;
             break;
         default:
-            display->current_time = (guint32) CurrentTime;
+#ifdef HAVE_XSYNC
+            if ((display->have_xsync) && (ev->type == display->xsync_event_base + XSyncAlarmNotify))
+            {
+                timestamp = ((XSyncAlarmNotifyEvent*) ev)->time;
+            }
+#endif /* HAVE_XSYNC */
             break;
     }
+
+    if ((timestamp == (guint32) CurrentTime) || TIMESTAMP_IS_BEFORE(display->current_time, timestamp))
+    {
+        display->current_time = timestamp;
+    }
+
     return display->current_time;
 }
 
 guint32
 myDisplayGetCurrentTime (DisplayInfo *display)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
 
     TRACE ("myDisplayGetCurrentTime gives timestamp=%u", (guint32) display->current_time);
     return display->current_time;
@@ -727,22 +751,22 @@ myDisplayGetCurrentTime (DisplayInfo *display)
 guint32
 myDisplayGetTime (DisplayInfo * display, guint32 timestamp)
 {
-    guint32 time;
+    guint32 display_timestamp;
 
-    time = timestamp;
-    if (time == (guint32) CurrentTime)
+    display_timestamp = timestamp;
+    if (display_timestamp == (guint32) CurrentTime)
     {
-        time = getXServerTime (display);
+        display_timestamp = getXServerTime (display);
     }
 
-    TRACE ("myDisplayGetTime gives timestamp=%u", (guint32) time);
-    return time;
+    TRACE ("myDisplayGetTime gives timestamp=%u", (guint32) display_timestamp);
+    return display_timestamp;
 }
 
 guint32
 myDisplayGetLastUserTime (DisplayInfo *display)
 {
-    g_return_val_if_fail (display != NULL, CurrentTime);
+    g_return_val_if_fail (display != NULL, (guint32) CurrentTime);
 
     TRACE ("myDisplayGetLastUserTime gives timestamp=%u", (guint32) display->last_user_time);
     return display->last_user_time;
