@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -312,7 +312,8 @@ struct _DisplayInfo
     gboolean have_xsync;
     gint shape_version;
     gint shape_event_base;
-    gint dbl_click_time;
+    gint double_click_time;
+    gint double_click_distance;
     gint xgrabcount;
     gint nb_screens;
     gchar* hostname;
@@ -360,6 +361,7 @@ struct _DisplayInfo
 DisplayInfo             *myDisplayInit                          (GdkDisplay *);
 DisplayInfo             *myDisplayClose                         (DisplayInfo *);
 gboolean                 myDisplayHaveShape                     (DisplayInfo *);
+gboolean                 myDisplayHaveShapeInput                (DisplayInfo *);
 gboolean                 myDisplayHaveRender                    (DisplayInfo *);
 Cursor                   myDisplayGetCursorBusy                 (DisplayInfo *);
 Cursor                   myDisplayGetCursorMove                 (DisplayInfo *);
