@@ -1,4 +1,4 @@
-/*      $Id: event_filter.c 26999 2008-05-31 14:35:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -127,9 +127,6 @@ default_event_filter (XEvent * xevent, gpointer data)
         case ColormapNotify:
             TRACE ("Unhandled ColormapNotify event");
             break;
-        case MappingNotify:
-            TRACE ("Unhandled MappingNotify event");
-            break;
         default:
             TRACE ("Unhandled Unknown event");
             break;
