@@ -1,4 +1,4 @@
-/*      $Id: event_filter.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,20 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
-#ifndef __EVENT_FILTER_H__
-#define __EVENT_FILTER_H__
+#ifndef INC_EVENT_FILTER_H
+#define INC_EVENT_FILTER_H
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -27,8 +33,10 @@
 
 /* this formatting is needed by glib-mkenums */
 typedef enum {
-    EVENT_FILTER_STOP = TRUE,
-    EVENT_FILTER_CONTINUE = FALSE
+    EVENT_FILTER_STOP     = 0x0,
+    EVENT_FILTER_PASS     = 0x0,
+    EVENT_FILTER_REMOVE   = 0x1,
+    EVENT_FILTER_CONTINUE = 0x1,
 }
 eventFilterStatus;
 
@@ -57,4 +65,4 @@ eventFilterStack        *eventFilterPop                         (eventFilterSetu
 eventFilterSetup        *eventFilterInit                        (gpointer);
 void                     eventFilterClose                       (eventFilterSetup *);
 
-#endif /* __EVENT_FILTER_H__ */
+#endif /* INC_EVENT_FILTER_H */
