@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -74,7 +74,7 @@
                                  (win == MYWINDOW_XWINDOW(c->buttons[SHADE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[STICK_BUTTON])))
 
-#define DBL_CLICK_GRAB          (ButtonMotionMask | \
+#define DOUBLE_CLICK_GRAB          (ButtonMotionMask | \
                                  PointerMotionMask | \
                                  ButtonPressMask | \
                                  ButtonReleaseMask)
@@ -123,17 +123,22 @@ struct _XfwmButtonClickData
     guint button;
     guint clicks;
     guint timeout;
-    gint x;
-    gint y;
-    gint xcurrent;
-    gint ycurrent;
+    gint  x0;
+    gint  y0;
+    guint t0;
+    gint  xcurrent;
+    gint  ycurrent;
+    guint tcurrent;
+    gint  double_click_time;
+    gint  double_click_distance;
     gboolean allow_double_click;
 };
 
 static gboolean
-typeOfClick_break (gpointer data)
+typeOfClick_end (gpointer data)
 {
     XfwmButtonClickData *passdata;
+    TRACE ("typeOfClick_end(): Exit typeOfClick() event loop");
 
     passdata = (XfwmButtonClickData *) data;
     if (passdata->timeout)
@@ -152,6 +157,7 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
 {
     XfwmButtonClickData *passdata;
     eventFilterStatus status;
+    guint32 timestamp;
     gboolean keep_going;
 
     keep_going = TRUE;
@@ -159,7 +165,9 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
     status = EVENT_FILTER_STOP;
 
     /* Update the display time */
-    myDisplayUpdateCurrentTime (passdata->display_info, xevent);
+    timestamp = myDisplayUpdateCurrentTime (passdata->display_info, xevent);
+    if (timestamp)
+        passdata->tcurrent = timestamp;
 
     if ((xevent->type == ButtonRelease) || (xevent->type == ButtonPress))
     {
@@ -194,12 +202,16 @@ typeOfClick_event_filter (XEvent * xevent, gpointer data)
         status = EVENT_FILTER_CONTINUE;
     }
 
-    if ((ABS (passdata->x - passdata->xcurrent) > 1) ||
-        (ABS (passdata->y - passdata->ycurrent) > 1) ||
+    if ((ABS (passdata->x0 - passdata->xcurrent) > passdata->double_click_distance) ||
+        (ABS (passdata->y0 - passdata->ycurrent) > passdata->double_click_distance) ||
+        (((gint) passdata->tcurrent - (gint) passdata->t0) > passdata->double_click_time) ||
         (!keep_going))
     {
-        TRACE ("event loop now finished");
-        typeOfClick_break (data);
+        TRACE ("click type=%u", passdata->clicks);
+        TRACE ("time=%ims (timeout=%ims)", (gint) passdata->tcurrent - (gint) passdata->t0, passdata->double_click_time);
+        TRACE ("dist x=%i (max=%i)", ABS (passdata->x0 - passdata->xcurrent), passdata->double_click_distance);
+        TRACE ("dist y=%i (max=%i)", ABS (passdata->y0 - passdata->ycurrent), passdata->double_click_distance);
+        typeOfClick_end (data);
     }
 
     return status;
@@ -218,7 +230,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
 
     display_info = screen_info->display_info;
     XFlush (display_info->dpy);
-    g = myScreenGrabPointer (screen_info, DBL_CLICK_GRAB, None, ev->xbutton.time);
+    g = myScreenGrabPointer (screen_info, DOUBLE_CLICK_GRAB, None, ev->xbutton.time);
 
     if (!g)
     {
@@ -231,15 +243,22 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     passdata.display_info = display_info;
     passdata.button = ev->xbutton.button;
     passdata.w = w;
-    passdata.x = ev->xbutton.x_root;
-    passdata.y = ev->xbutton.y_root;
-    passdata.xcurrent = passdata.x;
-    passdata.ycurrent = passdata.y;
+    passdata.x0 = ev->xbutton.x_root;
+    passdata.y0 = ev->xbutton.y_root;
+    passdata.t0 = ev->xbutton.time;
+    passdata.xcurrent = passdata.x0;
+    passdata.ycurrent = passdata.y0;
+    passdata.tcurrent = passdata.t0;
     passdata.clicks = 1;
     passdata.allow_double_click = allow_double_click;
-    passdata.timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
-                                           display_info->dbl_click_time,
-                                           (GSourceFunc) typeOfClick_break,
+    passdata.double_click_time = display_info->double_click_time;
+    passdata.double_click_distance = display_info->double_click_distance;
+    TRACE ("Double click time= %i, distance=%i\n", display_info->double_click_time,
+                                                   display_info->double_click_distance);
+    /* Use G_PRIORITY_DEFAULT_IDLE to make sure the timeout occurs after possible events */
+    passdata.timeout =  g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE,
+                                           display_info->double_click_time,
+                                           (GSourceFunc) typeOfClick_end,
                                            (gpointer) &passdata, NULL);
 
     TRACE ("entering typeOfClick loop");
@@ -523,6 +542,7 @@ static void
 edgeButton (Client * c, int part, XButtonEvent * ev)
 {
     ScreenInfo *screen_info;
+    XfwmButtonClickType tclick;
     guint state;
 
     screen_info = c->screen_info;
@@ -530,14 +550,12 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
 
     if (ev->button == Button2)
     {
-        XfwmButtonClickType tclick;
-
         tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, FALSE);
         if (tclick == XFWM_BUTTON_CLICK)
         {
             clientLower (c, None);
         }
-        else if (tclick != XFWM_BUTTON_UNDEFINED)
+        else if (tclick == XFWM_BUTTON_DRAG)
         {
             clientMove (c, (XEvent *) ev);
         }
@@ -553,7 +571,28 @@ edgeButton (Client * c, int part, XButtonEvent * ev)
             }
             clientRaise (c, None);
         }
-        clientResize (c, part, (XEvent *) ev);
+        tclick = typeOfClick (screen_info, c->window, (XEvent *) ev, TRUE);
+        if (tclick == XFWM_BUTTON_DOUBLE_CLICK)
+        {
+            switch (part)
+            {
+                case CORNER_COUNT + SIDE_LEFT:
+                case CORNER_COUNT + SIDE_RIGHT:
+                    clientFill(c, CLIENT_FILL_HORIZ);
+                    break;
+                case CORNER_COUNT + SIDE_TOP:
+                case CORNER_COUNT + SIDE_BOTTOM:
+                    clientFill(c, CLIENT_FILL_VERT);
+                    break;
+                default:
+                    clientFill(c, CLIENT_FILL);
+                    break;
+            }
+        }
+        else if (tclick == XFWM_BUTTON_DRAG)
+        {
+            clientResize (c, part, (XEvent *) ev);
+        }
     }
 }
 
@@ -662,16 +701,16 @@ button1Action (Client * c, XButtonEvent * ev)
     {
         switch (screen_info->params->double_click_action)
         {
-            case DBL_CLICK_ACTION_MAXIMIZE:
+            case DOUBLE_CLICK_ACTION_MAXIMIZE:
                 clientToggleMaximized (c, WIN_STATE_MAXIMIZED, TRUE);
                 break;
-            case DBL_CLICK_ACTION_SHADE:
+            case DOUBLE_CLICK_ACTION_SHADE:
                 clientToggleShaded (c);
                 break;
-            case DBL_CLICK_ACTION_FILL:
+            case DOUBLE_CLICK_ACTION_FILL:
                 clientFill(c, CLIENT_FILL);
                 break;
-            case DBL_CLICK_ACTION_HIDE:
+            case DOUBLE_CLICK_ACTION_HIDE:
                 if (CLIENT_CAN_HIDE_WINDOW (c))
                 {
                     clientWithdraw (c, c->win_workspace, TRUE);
@@ -750,7 +789,10 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
         }
         else if (!FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
-            clientShade (c);
+            if (screen_info->params->mousewheel_rollup)
+            {
+                clientShade (c);
+            }
         }
     }
     else if (ev->button == Button5)
@@ -762,7 +804,10 @@ titleButton (Client * c, guint state, XButtonEvent * ev)
         }
         else if (FLAG_TEST (c->flags, CLIENT_FLAG_SHADED))
         {
-            clientUnshade (c);
+            if (screen_info->params->mousewheel_rollup)
+            {
+                clientUnshade (c);
+            }
         }
     }
     else if (ev->button == Button6)
@@ -2134,19 +2179,24 @@ handleShape (DisplayInfo *display_info, XShapeEvent * ev)
     if (c)
     {
         update = FALSE;
-        if (ev->kind == ShapeBounding)
+        if (ev->kind == ShapeInput)
+        {
+            frameSetShapeInput (c);
+            update = TRUE;
+        }
+        else if (ev->kind == ShapeBounding)
         {
             if ((ev->shaped) && !FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
             {
-                FLAG_SET (c->flags, CLIENT_FLAG_HAS_SHAPE);
-                clientGetMWMHints (c, TRUE);
                 update = TRUE;
+                FLAG_SET (c->flags, CLIENT_FLAG_HAS_SHAPE);
+                clientGetMWMHints (c, update);
             }
             else if (!(ev->shaped) && FLAG_TEST (c->flags, CLIENT_FLAG_HAS_SHAPE))
             {
-                FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_SHAPE);
-                clientGetMWMHints (c, TRUE);
                 update = TRUE;
+                FLAG_UNSET (c->flags, CLIENT_FLAG_HAS_SHAPE);
+                clientGetMWMHints (c, update);
             }
         }
         if (!update)
@@ -2669,7 +2719,7 @@ set_reload (GObject * obj, GdkEvent * ev, gpointer data)
 }
 
 static gboolean
-dbl_click_time_cb (GObject * obj, GdkEvent * ev, gpointer data)
+double_click_time_cb (GObject * obj, GdkEvent * ev, gpointer data)
 {
     DisplayInfo *display_info;
     GValue tmp_val = { 0, };
@@ -2680,7 +2730,25 @@ dbl_click_time_cb (GObject * obj, GdkEvent * ev, gpointer data)
     g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
     {
-        display_info->dbl_click_time = abs (g_value_get_int (&tmp_val));
+        display_info->double_click_time = abs (g_value_get_int (&tmp_val));
+    }
+
+    return (TRUE);
+}
+
+static gboolean
+double_click_distance_cb (GObject * obj, GdkEvent * ev, gpointer data)
+{
+    DisplayInfo *display_info;
+    GValue tmp_val = { 0, };
+
+    display_info = (DisplayInfo *) data;
+    g_return_val_if_fail (display_info, TRUE);
+
+    g_value_init (&tmp_val, G_TYPE_INT);
+    if (gdk_setting_get ("gtk-double-click-distance", &tmp_val))
+    {
+        display_info->double_click_distance = abs (g_value_get_int (&tmp_val));
     }
 
     return (TRUE);
@@ -2717,6 +2785,10 @@ refresh_font_cb (GObject * obj, GdkEvent * ev, gpointer data)
     return (TRUE);
 }
 
+/*
+ * The size-changed signal is emitted when the pixel width or height
+ * of a screen changes.
+ */
 static void
 size_changed_cb(GdkScreen *gscreen, gpointer data)
 {
@@ -2740,22 +2812,71 @@ size_changed_cb(GdkScreen *gscreen, gpointer data)
         return;
     }
 
+    size_changed = myScreenComputeSize (screen_info);
+    if (size_changed)
+    {
+        myScreenInvalidateMonitorCache (screen_info);
+
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
+                        screen_info->width, screen_info->height, screen_info->margins);
+
+        placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
+
+        clientScreenResize (screen_info, FALSE);
+
+        compositorUpdateScreenSize (screen_info);
+    }
+}
+
+/*
+ * The monitors-changed signal is emitted when the number, size or
+ * position of the monitors attached to the screen change.
+ */
+static void
+monitors_changed_cb(GdkScreen *gscreen, gpointer data)
+{
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
+    gint previous_num_monitors;
+    gboolean size_changed;
+
+    TRACE ("entering monitors_changed_cb");
+
+    screen_info = (ScreenInfo *) data;
+    g_return_if_fail (screen_info);
+    display_info = screen_info->display_info;
+
+    if (gdk_screen_get_n_monitors (screen_info->gscr) == 0)
+    {
+        /*
+         * Recent Xorg drivers disable the output when the lid
+         * is closed, leaving no active monitor, in that case simply
+         * ignore the event to avoid messing with windows' positions.
+         */
+        return;
+    }
+
     /*
      * We have added/removed a monitor or even changed the layout,
      * the cache for monitor position we use in our screen structure
      * is not valid anymore and potentially refers to a monitor that
      * was just removed, so invalidate it.
      */
-    screen_info->cache_monitor.x = -1;
-    screen_info->cache_monitor.y = -1;
-    screen_info->cache_monitor.width = 0;
-    screen_info->cache_monitor.height = 0;
-
+    previous_num_monitors = screen_info->num_monitors;
+    myScreenInvalidateMonitorCache (screen_info);
+    myScreenRebuildMonitorIndex (screen_info);
     size_changed = myScreenComputeSize (screen_info);
-    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
-                    screen_info->width, screen_info->height, screen_info->margins);
-    placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
-    clientScreenResize (screen_info);
+
+    if (size_changed || (screen_info->num_monitors != previous_num_monitors))
+    {
+
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count,
+                        screen_info->width, screen_info->height, screen_info->margins);
+
+        placeSidewalks (screen_info, screen_info->params->wrap_workspaces);
+
+        clientScreenResize (screen_info, (screen_info->num_monitors < previous_num_monitors));
+    }
 
     if (size_changed)
     {
@@ -2763,24 +2884,6 @@ size_changed_cb(GdkScreen *gscreen, gpointer data)
     }
 }
 
-static void
-monitors_changed_cb(GdkScreen *gscreen, gpointer data)
-{
-    ScreenInfo *screen_info;
-
-    TRACE ("entering monitors_changed_cb");
-
-    screen_info = (ScreenInfo *) data;
-    g_return_if_fail (screen_info);
-
-    /*
-     * From the window manager point of view,
-     * a XRand 1.2 monitor change is similar to
-     * a screen size change.
-     */
-    size_changed_cb (gscreen, data);
-}
-
 void
 initGtkCallbacks (ScreenInfo *screen_info)
 {
@@ -2810,7 +2913,9 @@ initGtkCallbacks (ScreenInfo *screen_info)
         g_signal_connect (settings, "notify::gtk-font-name",
             G_CALLBACK (set_reload), (gpointer) (screen_info->display_info));
         g_signal_connect (settings, "notify::gtk-double-click-time",
-            G_CALLBACK (dbl_click_time_cb), (gpointer) (screen_info->display_info));
+            G_CALLBACK (double_click_time_cb), (gpointer) (screen_info->display_info));
+        g_signal_connect (settings, "notify::gtk-double-click-distance",
+            G_CALLBACK (double_click_distance_cb), (gpointer) (screen_info->display_info));
         g_signal_connect_after (settings, "notify::gtk-xft-antialias",
             G_CALLBACK (refresh_font_cb), (gpointer) (screen_info));
         g_signal_connect_after (settings, "notify::gtk-xft-dpi",
