@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2009 Olivier Fourdan,
+        xfwm4    - (c) 2002-2010 Olivier Fourdan,
                        2008 Jannis Pohlmann
 
  */
@@ -122,55 +122,50 @@ parseKeyString (Display * dpy, MyKey * key, const char *str)
 gboolean
 grabKey (Display * dpy, MyKey * key, Window w)
 {
-    gboolean status;
+    int status;
 
     TRACE ("entering grabKey");
 
     status = GrabSuccess;
     if (key->keycode)
     {
-        if (key->modifier == 0)
+        if (key->modifier != 0)
         {
-            status |=
-                XGrabKey (dpy, key->keycode, AnyModifier, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-        }
-        else
-        {
-            /* Here we grab all combinations of well known modifiers */
             status |=
                 XGrabKey (dpy, key->keycode,
                                         key->modifier, w,
                                         TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | ScrollLockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | NumLockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | LockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | ScrollLockMask | NumLockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | ScrollLockMask | LockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | LockMask | NumLockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
-            status |=
-                XGrabKey (dpy, key->keycode,
-                                        key->modifier | ScrollLockMask | LockMask | NumLockMask, w,
-                                        TRUE, GrabModeAsync, GrabModeSync);
         }
+
+        /* Here we grab all combinations of well known modifiers */
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | ScrollLockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | NumLockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | LockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | ScrollLockMask | NumLockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | ScrollLockMask | LockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | LockMask | NumLockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
+        status |=
+            XGrabKey (dpy, key->keycode,
+                                    key->modifier | ScrollLockMask | LockMask | NumLockMask, w,
+                                    TRUE, GrabModeAsync, GrabModeAsync);
     }
 
     return (status == GrabSuccess);
@@ -187,7 +182,7 @@ ungrabKeys (Display * dpy, Window w)
 gboolean
 grabButton (Display * dpy, int button, int modifier, Window w)
 {
-    gboolean status;
+    int status;
 
     TRACE ("entering grabButton");
 
@@ -277,9 +272,7 @@ initModifiers (Display * dpy)
 {
     XModifierKeymap *modmap;
     KeySym *keymap;
-    unsigned int keycode;
-    int min_keycode;
-    int max_keycode;
+    int min_keycode, max_keycode, keycode;
     int keysyms_per_keycode;
     int i;
 
