@@ -16,7 +16,7 @@
         MA 02110-1301, USA.
 
 
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -55,10 +55,16 @@ void                     clientUpdateLayerState                 (Client *);
 void                     clientSetNetActiveWindow               (ScreenInfo *,
                                                                  Client *,
                                                                  guint32);
-void                    clientRemoveNetWMPing                   (Client *);
-gboolean                clientSendNetWMPing                     (Client *,
+void                     clientHandleNetActiveWindow            (Client *,
+                                                                 guint32,
+                                                                 gboolean);
+void                     clientRemoveNetWMPing                  (Client *);
+gboolean                 clientSendNetWMPing                    (Client *,
                                                                  guint32);
-void                    clientReceiveNetWMPong                  (ScreenInfo *,
+void                     clientReceiveNetWMPong                 (ScreenInfo *,
                                                                  guint32);
+gboolean                 clientGetUserTime                      (Client *);
+void                     clientAddUserTimeWin                   (Client *);
+void                     clientRemoveUserTimeWin                (Client *);
 
 #endif /* INC_NETWM_H */
