@@ -1,4 +1,4 @@
-/*      $Id: placement.h 25778 2007-06-03 15:21:02Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,18 +12,19 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
-
 #ifndef INC_PLACEMENT_H
 #define INC_PLACEMENT_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <glib.h>
@@ -34,14 +35,16 @@
 #define CLIENT_CONSTRAINED_LEFT    1<<2
 #define CLIENT_CONSTRAINED_RIGHT   1<<3
 
-void                     clientMaxSpace                         (ScreenInfo *, 
-                                                                 int *, 
-                                                                 int *, 
-                                                                 int *, 
+void                     clientMaxSpace                         (ScreenInfo *,
+                                                                 int *,
+                                                                 int *,
+                                                                 int *,
                                                                  int *);
 gboolean                 clientCkeckTitle                       (Client *);
-unsigned int             clientConstrainPos                     (Client *, 
+unsigned int             clientConstrainPos                     (Client *,
                                                                  gboolean);
 void                     clientInitPosition                     (Client *);
+void                     clientFill                             (Client *,
+                                                                 int);
 
 #endif /* INC_PLACEMENT_H */
