@@ -1,4 +1,4 @@
-/*      $Id: screen.h 29514 2009-02-19 09:56:25Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2009 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -73,10 +73,10 @@ struct _ScreenInfo
     Client *last_raise;
     GList *windows;
     Client *clients;
-    unsigned int client_count;
+    guint client_count;
     unsigned long client_serial;
-    int key_grabs;
-    int pointer_grabs;
+    gint key_grabs;
+    gint pointer_grabs;
 
     /* Theme pixmaps and other params, per screen */
     XfwmColor title_colors[2];
@@ -93,33 +93,36 @@ struct _ScreenInfo
     GdkGC *white_gc;
 
     /* Title font height */
-    int font_height;
+    gint font_height;
 
     /* Screen data */
     Colormap cmap;
     GdkScreen *gscr;
     Screen *xscreen;
-    int depth;
-    int width;
-    int height;
+    gint depth;
+    gint width;
+    gint height;
     Visual *visual;
 
     GtkWidget *gtk_win;
     xfwmWindow sidewalk[4];
     Window xfwm4_win;
     Window xroot;
+    Window shape_win;
 
-    int gnome_margins[4];
-    int margins[4];
-    int screen;
-    int current_ws;
-    int previous_ws;
+    gint gnome_margins[4];
+    gint margins[4];
+    gint screen;
+    guint current_ws;
+    guint previous_ws;
 
     /* Monitor search caching */
     GdkRectangle cache_monitor;
+    gint num_monitors;
+    GArray *monitors_index;
 
     /* Workspace definitions */
-    int workspace_count;
+    guint workspace_count;
     gchar **workspace_names;
     int workspace_names_items;
     NetWmDesktopLayout desktop_layout;
@@ -190,7 +193,6 @@ ScreenInfo              *myScreenInit                           (DisplayInfo *,
 ScreenInfo              *myScreenClose                          (ScreenInfo *);
 Display                 *myScreenGetXDisplay                    (ScreenInfo *);
 GtkWidget               *myScreenGetGtkWidget                   (ScreenInfo *);
-GtkWidget               *myScreenGetGtkWidget                   (ScreenInfo *);
 GdkWindow               *myScreenGetGdkWindow                   (ScreenInfo *);
 gboolean                 myScreenGrabKeyboard                   (ScreenInfo *,
                                                                  guint32);
@@ -213,6 +215,11 @@ int                      myScreenGetKeyPressed                  (ScreenInfo *,
 Client                  *myScreenGetClientFromWindow            (ScreenInfo *,
                                                                  Window,
                                                                  unsigned short);
+gboolean                 myScreenComputeSize                    (ScreenInfo *);
+gint                     myScreenGetNumMonitors                 (ScreenInfo *);
+gint                     myScreenGetMonitorIndex                (ScreenInfo *,
+                                                                 gint);
+gboolean                 myScreenRebuildMonitorIndex            (ScreenInfo *);
 void                     myScreenInvalidateMonitorCache         (ScreenInfo *);
 void                     myScreenFindMonitorAtPoint             (ScreenInfo *,
                                                                  gint,
