@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2009 Olivier Fourdan,
+        xfwm4    - (c) 2002-2010 Olivier Fourdan,
                        2008 Jannis Pohlmann
 
  */
@@ -439,11 +439,22 @@ loadTheme (ScreenInfo *screen_info, Settings *rc)
     colsym[XPM_COLOR_SYMBOL_SIZE].name = NULL;
     colsym[XPM_COLOR_SYMBOL_SIZE].value = NULL;
 
-    display_info->dbl_click_time = abs (getIntValue ("dbl_click_time", rc));
+    /* Standard double click time ... */
+    display_info->double_click_time = abs (getIntValue ("double_click_time", rc));
     g_value_init (&tmp_val, G_TYPE_INT);
     if (gdk_setting_get ("gtk-double-click-time", &tmp_val))
     {
-        display_info->dbl_click_time = abs (g_value_get_int (&tmp_val));
+        display_info->double_click_time = abs (g_value_get_int (&tmp_val));
+        g_value_unset (&tmp_val);
+    }
+
+    /* ... and distance */
+    display_info->double_click_distance = abs (getIntValue ("double_click_distance", rc));
+    g_value_init (&tmp_val, G_TYPE_INT);
+    if (gdk_setting_get ("gtk-double-click-distance", &tmp_val))
+    {
+        display_info->double_click_distance = abs (g_value_get_int (&tmp_val));
+        g_value_unset (&tmp_val);
     }
 
     screen_info->font_height = 0;
@@ -668,7 +679,8 @@ loadSettings (ScreenInfo *screen_info)
         {"cycle_hidden", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_minimum", NULL, G_TYPE_BOOLEAN, TRUE},
         {"cycle_workspaces", NULL, G_TYPE_BOOLEAN, TRUE},
-        {"dbl_click_time", NULL, G_TYPE_INT, TRUE},
+        {"double_click_time", NULL, G_TYPE_INT, TRUE},
+        {"double_click_distance", NULL, G_TYPE_INT, TRUE},
         {"double_click_action", NULL, G_TYPE_STRING, TRUE},
         {"easy_click", NULL, G_TYPE_STRING, TRUE},
         {"focus_hint", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -685,6 +697,7 @@ loadSettings (ScreenInfo *screen_info)
         {"placement_ratio", NULL, G_TYPE_INT, TRUE},
         {"placement_mode", NULL, G_TYPE_STRING, TRUE},
         {"popup_opacity", NULL, G_TYPE_INT, TRUE},
+        {"mousewheel_rollup", NULL, G_TYPE_BOOLEAN, TRUE},
         {"prevent_focus_stealing", NULL, G_TYPE_BOOLEAN, TRUE},
         {"raise_delay", NULL, G_TYPE_INT, TRUE},
         {"raise_on_click", NULL, G_TYPE_BOOLEAN, TRUE},
@@ -759,12 +772,14 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("focus_hint", rc);
     screen_info->params->focus_new =
         getBoolValue ("focus_new", rc);
-    screen_info->params->raise_on_focus =
-        getBoolValue ("raise_on_focus", rc);
+    screen_info->params->mousewheel_rollup =
+        getBoolValue ("mousewheel_rollup", rc);
     screen_info->params->prevent_focus_stealing =
         getBoolValue ("prevent_focus_stealing", rc);
     screen_info->params->raise_delay =
         getIntValue ("raise_delay", rc);
+    screen_info->params->raise_on_focus =
+        getBoolValue ("raise_on_focus", rc);
     screen_info->params->focus_delay =
         getIntValue ("focus_delay", rc);
     screen_info->params->raise_on_click =
@@ -843,23 +858,23 @@ loadSettings (ScreenInfo *screen_info)
     value = getStringValue ("double_click_action", rc);
     if (!g_ascii_strcasecmp ("shade", value))
     {
-        screen_info->params->double_click_action = DBL_CLICK_ACTION_SHADE;
+        screen_info->params->double_click_action = DOUBLE_CLICK_ACTION_SHADE;
     }
     else if (!g_ascii_strcasecmp ("hide", value))
     {
-        screen_info->params->double_click_action = DBL_CLICK_ACTION_HIDE;
+        screen_info->params->double_click_action = DOUBLE_CLICK_ACTION_HIDE;
     }
     else if (!g_ascii_strcasecmp ("maximize", value))
     {
-        screen_info->params->double_click_action = DBL_CLICK_ACTION_MAXIMIZE;
+        screen_info->params->double_click_action = DOUBLE_CLICK_ACTION_MAXIMIZE;
     }
     else if (!g_ascii_strcasecmp ("fill", value))
     {
-        screen_info->params->double_click_action = DBL_CLICK_ACTION_FILL;
+        screen_info->params->double_click_action = DOUBLE_CLICK_ACTION_FILL;
     }
     else
     {
-        screen_info->params->double_click_action = DBL_CLICK_ACTION_NONE;
+        screen_info->params->double_click_action = DOUBLE_CLICK_ACTION_NONE;
     }
 
     if (screen_info->workspace_count == 0)
@@ -1173,7 +1188,8 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 }
                 else if ((!strcmp (name, "button_offset"))
                       || (!strcmp (name, "button_spacing"))
-                      || (!strcmp (name, "dbl_click_time"))
+                      || (!strcmp (name, "double_click_time"))
+                      || (!strcmp (name, "double_DOUBLE_CLICKclick_distance"))
                       || (!strcmp (name, "maximized_offset"))
                       || (!strcmp (name, "shadow_delta_height"))
                       || (!strcmp (name, "shadow_delta_width"))
@@ -1272,6 +1288,27 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->focus_hint = g_value_get_boolean (value);
                 }
+                else if (!strcmp (name, "mousewheel_rollup"))
+                {
+                    screen_info->params->mousewheel_rollup = g_value_get_boolean (value);
+                }
+                else if (!strcmp (name, "prevent_focus_stealing"))
+                {
+                    screen_info->params->prevent_focus_stealing = g_value_get_boolean (value);
+                }
+                else if (!strcmp (name, "raise_with_any_button"))
+                {
+                    screen_info->params->raise_with_any_button = g_value_get_boolean (value);
+                    update_grabs (screen_info);
+                }
+                else if (!strcmp (name, "restore_on_move"))
+                {
+                    screen_info->params->restore_on_move = g_value_get_boolean (value);
+                }
+                else if (!strcmp (name, "scroll_workspaces"))
+                {
+                    screen_info->params->scroll_workspaces = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "show_dock_shadow"))
                 {
                     screen_info->params->show_dock_shadow = g_value_get_boolean (value);
@@ -1291,23 +1328,6 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->snap_resist = g_value_get_boolean (value);
                 }
-                else if (!strcmp (name, "prevent_focus_stealing"))
-                {
-                    screen_info->params->prevent_focus_stealing = g_value_get_boolean (value);
-                }
-                else if (!strcmp (name, "raise_with_any_button"))
-                {
-                    screen_info->params->raise_with_any_button = g_value_get_boolean (value);
-                    update_grabs (screen_info);
-                }
-                else if (!strcmp (name, "restore_on_move"))
-                {
-                    screen_info->params->restore_on_move = g_value_get_boolean (value);
-                }
-                else if (!strcmp (name, "scroll_workspaces"))
-                {
-                    screen_info->params->scroll_workspaces = g_value_get_boolean (value);
-                }
                 else if (!strcmp (name, "toggle_workspaces"))
                 {
                     screen_info->params->toggle_workspaces = g_value_get_boolean (value);
