@@ -1,4 +1,4 @@
-/*      $Id: settings.h 27421 2008-08-04 20:28:44Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2008 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -56,6 +56,7 @@ enum
     KEY_ADD_WORKSPACE,
     KEY_CLOSE_WINDOW,
     KEY_CYCLE_WINDOWS,
+    KEY_CYCLE_REVERSE_WINDOWS,
     KEY_DEL_ACTIVE_WORKSPACE,
     KEY_DEL_WORKSPACE,
     KEY_DOWN_WORKSPACE,
@@ -96,6 +97,8 @@ enum
     KEY_SHADE_WINDOW,
     KEY_SHOW_DESKTOP,
     KEY_STICK_WINDOW,
+    KEY_SWITCH_APPLICATION,
+    KEY_SWITCH_WINDOW,
     KEY_TOGGLE_ABOVE,
     KEY_TOGGLE_FULLSCREEN,
     KEY_UP_WORKSPACE,
@@ -130,11 +133,11 @@ enum
 
 enum
 {
-    DBL_CLICK_ACTION_NONE,
-    DBL_CLICK_ACTION_MAXIMIZE,
-    DBL_CLICK_ACTION_SHADE,
-    DBL_CLICK_ACTION_FILL,
-    DBL_CLICK_ACTION_HIDE
+    DOUBLE_CLICK_ACTION_NONE,
+    DOUBLE_CLICK_ACTION_MAXIMIZE,
+    DOUBLE_CLICK_ACTION_SHADE,
+    DOUBLE_CLICK_ACTION_FILL,
+    DOUBLE_CLICK_ACTION_HIDE
 };
 
 enum
@@ -162,27 +165,28 @@ struct _XfwmParams
 {
     MyKey keys[KEY_COUNT];
     gchar button_layout[BUTTON_STRING_COUNT + 1];
-    unsigned int xfwm_margins[4];
+    int xfwm_margins[4];
     int activate_action;
     int button_offset;
     int button_spacing;
     int double_click_action;
-    int easy_click;
+    guint easy_click;
+    int focus_delay;
     int frame_opacity;
     int inactive_opacity;
     int maximized_offset;
     int move_opacity;
-    int placement_ratio;
     int placement_mode;
+    int placement_ratio;
     int popup_opacity;
     int raise_delay;
-    int focus_delay;
     int resize_opacity;
     int restore_on_move;
     int shadow_delta_height;
     int shadow_delta_width;
     int shadow_delta_x;
     int shadow_delta_y;
+    int shadow_opacity;
     int snap_width;
     int title_alignment;
     int title_horizontal_offset;
@@ -192,28 +196,33 @@ struct _XfwmParams
     gboolean box_move;
     gboolean box_resize;
     gboolean click_to_focus;
+    gboolean cycle_apps_only;
+    gboolean cycle_draw_frame;
     gboolean cycle_hidden;
     gboolean cycle_minimum;
     gboolean cycle_workspaces;
     gboolean focus_hint;
     gboolean focus_new;
     gboolean full_width_title;
+    gboolean mousewheel_rollup;
     gboolean prevent_focus_stealing;
     gboolean raise_on_click;
     gboolean raise_on_focus;
     gboolean raise_with_any_button;
+    gboolean repeat_urgent_blink;
     gboolean scroll_workspaces;
     gboolean show_app_icon;
     gboolean show_dock_shadow;
     gboolean show_frame_shadow;
     gboolean show_popup_shadow;
+    gboolean snap_resist;
     gboolean snap_to_border;
     gboolean snap_to_windows;
-    gboolean snap_resist;
     gboolean title_vertical_offset_active;
     gboolean title_vertical_offset_inactive;
     gboolean toggle_workspaces;
     gboolean unredirect_overlays;
+    gboolean urgent_blink;
     gboolean use_compositing;
     gboolean wrap_cycle;
     gboolean wrap_layout;
