@@ -1,4 +1,4 @@
-/*      $Id: stacking.c 25175 2007-03-15 22:29:49Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,12 +12,18 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
@@ -34,6 +40,8 @@
 #include "frame.h"
 #include "focus.h"
 
+static guint raise_timeout = 0;
+
 void
 clientApplyStackList (ScreenInfo *screen_info)
 {
@@ -53,12 +61,12 @@ clientApplyStackList (ScreenInfo *screen_info)
 
     if (nwindows)
     {
-        GList *index;
+        GList *list;
         Client *c = NULL;
 
-        for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
+        for (list = g_list_last(screen_info->windows_stack); list; list = g_list_previous (list))
         {
-            c = (Client *) index->data;
+            c = (Client *) list->data;
             xwinstack[i++] = c->frame;
             DBG ("  [%i] \"%s\" (0x%lx)", i, c->name, c->window);
         }
@@ -74,7 +82,7 @@ clientGetLowestTransient (Client * c)
 {
     ScreenInfo *screen_info;
     Client *lowest_transient, *c2;
-    GList *index;
+    GList *list;
 
     g_return_val_if_fail (c != NULL, NULL);
 
@@ -82,9 +90,9 @@ clientGetLowestTransient (Client * c)
 
     lowest_transient = NULL;
     screen_info = c->screen_info;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c2 = (Client *) index->data;
+        c2 = (Client *) list->data;
         if ((c2 != c) && clientIsTransientFor (c2, c))
         {
             lowest_transient = c2;
@@ -100,16 +108,16 @@ clientGetHighestTransientOrModalFor (Client * c)
     ScreenInfo *screen_info;
     Client *highest_transient;
     Client *c2;
-    GList *index;
+    GList *list;
 
     g_return_val_if_fail (c != NULL, NULL);
     TRACE ("entering clientGetHighestTransientOrModalFor");
 
     screen_info = c->screen_info;
     highest_transient = NULL;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c2 = (Client *) index->data;
+        c2 = (Client *) list->data;
         if (c2)
         {
             if (clientIsTransientOrModalFor (c, c2))
@@ -128,16 +136,16 @@ clientGetTopMostForGroup (Client * c)
     ScreenInfo *screen_info;
     Client *top_most;
     Client *c2;
-    GList *index;
+    GList *list;
 
     g_return_val_if_fail (c != NULL, NULL);
     TRACE ("entering clientGetTopMostForGroup");
 
     screen_info = c->screen_info;
     top_most = NULL;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c2 = (Client *) index->data;
+        c2 = (Client *) list->data;
         if (c2 != c)
         {
             if (clientSameGroup (c, c2))
@@ -154,7 +162,7 @@ gboolean
 clientIsTopMost (Client *c)
 {
     ScreenInfo *screen_info;
-    GList *index, *index2;
+    GList *list, *list2;
     Client *c2;
 
     g_return_val_if_fail (c != NULL, FALSE);
@@ -162,35 +170,35 @@ clientIsTopMost (Client *c)
 
     screen_info = c->screen_info;
 
-    index = g_list_find (screen_info->windows_stack, (gconstpointer) c);
-    if (index)
+    list = g_list_find (screen_info->windows_stack, (gconstpointer) c);
+    if (list)
     {
-        index2 = g_list_next (index);
-        while (index2)
+        list2 = g_list_next (list);
+        while (list2)
         {
-            c2 = (Client *) index2->data;
+            c2 = (Client *) list2->data;
             if (FLAG_TEST (c2->xfwm_flags, XFWM_FLAG_VISIBLE) && (c2->win_layer == c->win_layer))
             {
                 return FALSE;
             }
-            index2 = g_list_next (index2);
+            list2 = g_list_next (list2);
         }
     }
     return TRUE;
 }
 
 Client *
-clientGetNextTopMost (ScreenInfo *screen_info, int layer, Client * exclude)
+clientGetNextTopMost (ScreenInfo *screen_info, guint layer, Client * exclude)
 {
     Client *top, *c;
-    GList *index;
+    GList *list;
 
     TRACE ("entering clientGetNextTopMost");
 
     top = NULL;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c = (Client *) index->data;
+        c = (Client *) list->data;
         TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name, c->window, (int) c->win_layer);
         if (!exclude || (c != exclude))
         {
@@ -206,17 +214,17 @@ clientGetNextTopMost (ScreenInfo *screen_info, int layer, Client * exclude)
 }
 
 Client *
-clientGetBottomMost (ScreenInfo *screen_info, int layer, Client * exclude)
+clientGetBottomMost (ScreenInfo *screen_info, guint layer, Client * exclude)
 {
     Client *bot, *c;
-    GList *index;
+    GList *list;
 
     TRACE ("entering clientGetBottomMost");
 
     bot = NULL;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c = (Client *) index->data;
+        c = (Client *) list->data;
         if (c)
         {
             TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name,
@@ -242,22 +250,22 @@ clientGetBottomMost (ScreenInfo *screen_info, int layer, Client * exclude)
  * conditions caused by querying the X server
  */
 Client *
-clientAtPosition (ScreenInfo *screen_info, int x, int y, Client * exclude)
+clientAtPosition (ScreenInfo *screen_info, int x, int y, GList * exclude_list)
 {
-    GList *index;
+    GList *list;
     Client *c, *c2;
 
     TRACE ("entering clientAtPosition");
 
     c = NULL;
-    for (index = g_list_last (screen_info->windows_stack); index; index = g_list_previous (index))
+    for (list = g_list_last (screen_info->windows_stack); list; list = g_list_previous (list))
     {
-        c2 = (Client *) index->data;
-        if ((frameX (c2) < x) && (frameX (c2) + frameWidth (c2) > x)
-            && (frameY (c2) < y) && (frameY (c2) + frameHeight (c2) > y))
+        c2 = (Client *) list->data;
+        if ((frameX (c2) <= x) && (frameX (c2) + frameWidth (c2) >= x)
+            && (frameY (c2) <= y) && (frameY (c2) + frameHeight (c2) >= y))
         {
-            if (clientSelectMask (c2, INCLUDE_SKIP_PAGER | INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
-                && (c2 != exclude))
+            if (clientSelectMask (c2, NULL, SEARCH_INCLUDE_SKIP_PAGER | SEARCH_INCLUDE_SKIP_TASKBAR, WINDOW_REGULAR_FOCUSABLE)
+                && !g_list_find (exclude_list, (gconstpointer) c2))
             {
                 c = c2;
                 break;
@@ -276,7 +284,7 @@ clientRaise (Client * c, Window wsibling)
     Client *c2, *c3, *client_sibling;
     GList *transients;
     GList *sibling;
-    GList *index1, *index2;
+    GList *list1, *list2;
     GList *windows_stack_copy;
 
     g_return_if_fail (c != NULL);
@@ -328,16 +336,16 @@ clientRaise (Client * c, Window wsibling)
         /* Search for the window that will be just on top of the raised window  */
         if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
                 if (sibling)
                 {
-                    index1 = g_list_next (sibling);
-                    if (index1)
+                    list1 = g_list_next (sibling);
+                    if (list1)
                     {
-                        client_sibling = (Client *) index1->data;
+                        client_sibling = (Client *) list1->data;
                         /* Do not place window under higher layers though */
                         if ((client_sibling) && (client_sibling->win_layer < c->win_layer))
                         {
@@ -365,9 +373,9 @@ clientRaise (Client * c, Window wsibling)
             screen_info->windows_stack = g_list_append (screen_info->windows_stack, c);
         }
         /* Now, look for transients, transients of transients, etc. */
-        for (index1 = windows_stack_copy; index1; index1 = g_list_next (index1))
+        for (list1 = windows_stack_copy; list1; list1 = g_list_next (list1))
         {
-            c2 = (Client *) index1->data;
+            c2 = (Client *) list1->data;
             if (c2)
             {
                 if ((c2 != c) && clientIsTransientOrModalFor (c2, c) && (c2->win_layer <= c->win_layer))
@@ -392,9 +400,9 @@ clientRaise (Client * c, Window wsibling)
                 }
                 else
                 {
-                    for (index2 = transients; index2; index2 = g_list_next (index2))
+                    for (list2 = transients; list2; list2 = g_list_next (list2))
                     {
-                        c3 = (Client *) index2->data;
+                        c3 = (Client *) list2->data;
                         if ((c3 != c2) && clientIsTransientOrModalFor (c2, c3))
                         {
                             transients = g_list_append (transients, c2);
@@ -444,7 +452,7 @@ clientLower (Client * c, Window wsibling)
     DisplayInfo *display_info;
     Client *c2, *client_sibling;
     GList *sibling;
-    GList *index;
+    GList *list;
     gint position;
 
     g_return_if_fail (c != NULL);
@@ -475,16 +483,16 @@ clientLower (Client * c, Window wsibling)
         }
         else if (wsibling)
         {
-            c2 = myDisplayGetClientFromWindow (display_info, wsibling, ANY);
+            c2 = myDisplayGetClientFromWindow (display_info, wsibling, SEARCH_FRAME | SEARCH_WINDOW);
             if (c2)
             {
                 sibling = g_list_find (screen_info->windows_stack, (gconstpointer) c2);
                 if (sibling)
                 {
-                    index = g_list_previous (sibling);
-                    if (index)
+                    list = g_list_previous (sibling);
+                    if (list)
                     {
-                        client_sibling = (Client *) index->data;
+                        client_sibling = (Client *) list->data;
                         /* Do not place window above lower layers though */
                         if ((client_sibling) && (client_sibling->win_layer > c->win_layer))
                         {
@@ -541,11 +549,11 @@ clientAdjustFullscreenLayer (Client *c, gboolean set)
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
         {
-            clientSetLayer (c, WIN_LAYER_ABOVE_DOCK);
+            clientSetLayer (c, WIN_LAYER_FULLSCREEN);
             return TRUE;
         }
     }
-    else if (c->win_layer == WIN_LAYER_ABOVE_DOCK)
+    else if (c->win_layer == WIN_LAYER_FULLSCREEN)
     {
         if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
@@ -682,3 +690,44 @@ clientClearLastRaise (ScreenInfo *screen_info)
     g_return_if_fail (screen_info);
     screen_info->last_raise = NULL;
 }
+
+static gboolean
+delayed_raise_cb (gpointer data)
+{
+    Client *c;
+
+    TRACE ("entering delayed_raise_cb");
+
+    clientClearDelayedRaise ();
+    c = clientGetFocus ();
+
+    if (c)
+    {
+        clientRaise (c, None);
+    }
+    return (TRUE);
+}
+
+void
+clientClearDelayedRaise (void)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+        raise_timeout = 0;
+    }
+}
+
+void
+clientResetDelayedRaise (ScreenInfo *screen_info)
+{
+    if (raise_timeout)
+    {
+        g_source_remove (raise_timeout);
+    }
+    raise_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
+                                        screen_info->params->raise_delay,
+                                        (GSourceFunc) delayed_raise_cb,
+                                        NULL, NULL);
+}
+
