@@ -1,4 +1,4 @@
-/*      $Id: stacking.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,9 +12,11 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -22,7 +24,7 @@
 #define INC_STACKING_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #include <glib.h>
@@ -34,21 +36,21 @@ Client                  *clientGetLowestTransient               (Client *);
 Client                  *clientGetHighestTransientOrModalFor    (Client *);
 gboolean                 clientIsTopMost                        (Client *);
 Client                  *clientGetTopMostForGroup               (Client *);
-Client                  *clientGetNextTopMost                   (ScreenInfo *, 
-                                                                 int, 
+Client                  *clientGetNextTopMost                   (ScreenInfo *,
+                                                                 guint,
                                                                  Client *);
-Client                  *clientGetBottomMost                    (ScreenInfo *, 
-                                                                 int, 
+Client                  *clientGetBottomMost                    (ScreenInfo *,
+                                                                 guint,
                                                                  Client *);
-Client                  *clientAtPosition                       (ScreenInfo *, 
-                                                                 int, 
-                                                                 int, 
-                                                                 Client *);
-void                     clientRaise                            (Client *, 
+Client                  *clientAtPosition                       (ScreenInfo *,
+                                                                 int,
+                                                                 int,
+                                                                 GList *);
+void                     clientRaise                            (Client *,
                                                                  Window);
-void                     clientLower                            (Client *, 
+void                     clientLower                            (Client *,
                                                                  Window);
-gboolean                 clientAdjustFullscreenLayer            (Client *, 
+gboolean                 clientAdjustFullscreenLayer            (Client *,
                                                                  gboolean);
 void                     clientAddToList                        (Client *);
 void                     clientRemoveFromList                   (Client *);
@@ -56,5 +58,7 @@ GList                   *clientGetStackList                     (ScreenInfo *);
 void                     clientSetLastRaise                     (Client *);
 Client                  *clientGetLastRaise                     (ScreenInfo *);
 void                     clientClearLastRaise                   (ScreenInfo *);
+void                     clientClearDelayedRaise                (void);
+void                     clientResetDelayedRaise                (ScreenInfo *);
 
 #endif /* INC_STACKING_H */
