@@ -1,4 +1,4 @@
-/*      $Id: startup_notification.h 23704 2006-11-04 22:36:42Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,10 +12,12 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
+
 
         Metacity - (c) 2003 Havoc Pennington
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
@@ -23,7 +25,7 @@
 #define INC_STARTUP_NOTIFICATION_H
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#include "config.h"
 #endif
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
@@ -47,4 +49,5 @@ void                     sn_process_event                       (XEvent *);
 #define                  sn_process_event(e) ;
 
 #endif /* HAVE_LIBSTARTUP_NOTIFICATION */
+
 #endif /* INC_STARTUP_NOTIFICATION_H */
