@@ -1,4 +1,4 @@
-/*      $Id: transients.c 25620 2007-04-20 21:30:53Z olivier $
+/*      $Id$
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -12,14 +12,16 @@
 
         You should have received a copy of the GNU General Public License
         along with this program; if not, write to the Free Software
-        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+        Foundation, Inc., Inc., 51 Franklin Street, Fifth Floor, Boston,
+        MA 02110-1301, USA.
 
-        xfwm4    - (c) 2002-2006 Olivier Fourdan
+
+        xfwm4    - (c) 2002-2010 Olivier Fourdan
 
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <glib.h>
@@ -39,7 +41,7 @@ clientGetTransient (Client * c)
 
     if ((c->transient_for) && (c->transient_for != c->screen_info->xroot))
     {
-        return clientGetFromWindow (c->screen_info, c->transient_for, WINDOW);
+        return myScreenGetClientFromWindow (c->screen_info, c->transient_for, SEARCH_WINDOW);
     }
     return NULL;
 }
@@ -61,9 +63,9 @@ clientIsModal (Client * c)
     g_return_val_if_fail (c != NULL, FALSE);
 
     TRACE ("entering clientIsModal");
-    /* 
-       If the WM_TRANSIENT_FOR hint is set to another toplevel window, the dialog is modal for that window; 
-       if WM_TRANSIENT_FOR is not set or set to the root window the dialog is modal for its window group. 
+    /*
+       If the WM_TRANSIENT_FOR hint is set to another toplevel window, the dialog is modal for that window;
+       if WM_TRANSIENT_FOR is not set or set to the root window the dialog is modal for its window group.
      */
     return (FLAG_TEST (c->flags, CLIENT_FLAG_STATE_MODAL) && (c->type & WINDOW_REGULAR_FOCUSABLE) &&
             (((c->transient_for != c->screen_info->xroot) && (c->transient_for != None) && (c->transient_for != c->window)) ||
@@ -95,6 +97,21 @@ clientSameGroup (Client * c1, Client * c2)
              (c2->group_leader == c1->window)));
 }
 
+gboolean
+clientSameLeader (Client * c1, Client * c2)
+{
+    g_return_val_if_fail (c1 != NULL, FALSE);
+    g_return_val_if_fail (c2 != NULL, FALSE);
+
+    TRACE ("entering clientSameLeader");
+
+    return ((c1 != c2) &&
+            (((c1->client_leader != None) &&
+              (c1->client_leader == c2->client_leader)) ||
+             (c1->client_leader == c2->window) ||
+             (c2->client_leader == c1->window)));
+}
+
 gboolean
 clientIsTransientFor (Client * c1, Client * c2)
 {
@@ -110,8 +127,8 @@ clientIsTransientFor (Client * c1, Client * c2)
             return (c1->transient_for == c2->window);
         }
         /*
-         * Transients for group shouldn't apply to other transients, or 
-         * it breaks stacking for some apps, noticeably mozilla "save as" 
+         * Transients for group shouldn't apply to other transients, or
+         * it breaks stacking for some apps, noticeably mozilla "save as"
          * dialog...
          */
         else if (c2->transient_for == None)
@@ -132,7 +149,13 @@ clientIsModalFor (Client * c1, Client * c2)
 
     if (FLAG_TEST (c1->flags, CLIENT_FLAG_STATE_MODAL) && (c1->type & WINDOW_REGULAR_FOCUSABLE) && (c1->serial >= c2->serial))
     {
-        return (clientIsTransientFor (c1, c2) || clientSameGroup (c1, c2));
+        /*
+         * We used to consider all windows of the same group here
+         * (ie. "clientSameGroup (c1, c2)") but that seems too
+         * wide so we restric modality to transient relationship
+         * for now.
+         */
+        return (clientIsTransientFor (c1, c2));
     }
     return FALSE;
 }
@@ -188,12 +211,12 @@ clientIsValidTransientOrModal (Client * c)
     if (clientIsTransientOrModalForGroup (c))
     {
         ScreenInfo *screen_info = c->screen_info;
-        GList *index;
+        GList *list;
 
         /* Look for a valid transient or modal for the same group */
-        for (index = screen_info->windows_stack; index; index = g_list_next (index))
+        for (list = screen_info->windows_stack; list; list = g_list_next (list))
         {
-            Client *c2 = (Client *) index->data;
+            Client *c2 = (Client *) list->data;
             if (c2 != c)
             {
                 if (clientIsTransientOrModalFor (c, c2))
@@ -213,10 +236,10 @@ clientIsValidTransientOrModal (Client * c)
 }
 
 gboolean
-clientTransientOrModalHasAncestor (Client * c, int ws)
+clientTransientOrModalHasAncestor (Client * c, guint ws)
 {
     Client *c2;
-    GList *index;
+    GList *list;
     ScreenInfo *screen_info;
 
     g_return_val_if_fail (c != NULL, FALSE);
@@ -229,9 +252,9 @@ clientTransientOrModalHasAncestor (Client * c, int ws)
     }
 
     screen_info = c->screen_info;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
+    for (list = screen_info->windows_stack; list; list = g_list_next (list))
     {
-        c2 = (Client *) index->data;
+        c2 = (Client *) list->data;
         if ((c2 != c) && !clientIsTransientOrModal (c2)
             && clientIsTransientOrModalFor (c, c2)
             && !FLAG_TEST (c2->flags, CLIENT_FLAG_ICONIFIED)
@@ -248,49 +271,57 @@ Client *
 clientGetModalFor (Client * c)
 {
     ScreenInfo *screen_info;
-    Client *latest_modal;
-    Client *c2, *c3;
-    GList *modals;
-    GList *index1, *index2;
+    Client *c2;
+    GList *list;
 
     g_return_val_if_fail (c != NULL, NULL);
     TRACE ("entering clientGetModalFor");
 
-    modals = NULL;
-    latest_modal = NULL;
     screen_info = c->screen_info;
-    for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
+    for (list = g_list_last(screen_info->windows_stack); list; list = g_list_previous (list))
     {
-        c2 = (Client *) index1->data;
+        c2 = (Client *) list->data;
         if (c2)
         {
             if ((c2 != c) && clientIsModalFor (c2, c))
             {
-                modals = g_list_append (modals, c2);
-                latest_modal = c2;
-            }
-            else
-            {
-                for (index2 = modals; index2;
-                    index2 = g_list_next (index2))
-                {
-                    c3 = (Client *) index2->data;
-                    if ((c3 != c2) && clientIsModalFor (c2, c3))
-                    {
-                        modals = g_list_append (modals, c2);
-                        latest_modal = c2;
-                        break;
-                    }
-                }
+                return c2;
             }
         }
     }
-    if (modals)
+    return NULL;
+}
+
+Client *
+clientGetTransientFor (Client * c)
+{
+    ScreenInfo *screen_info;
+    Client *latest_transient;
+    Client *c2;
+    GList *list;
+
+    g_return_val_if_fail (c != NULL, NULL);
+    TRACE ("entering clientGetTransientFor");
+
+    latest_transient = c;
+    screen_info = c->screen_info;
+    for (list = g_list_last(screen_info->windows_stack); list; list = g_list_previous (list))
     {
-        g_list_free (modals);
+        if (!clientIsTransient (latest_transient))
+        {
+            break;
+        }
+        c2 = (Client *) list->data;
+        if (c2)
+        {
+            if (clientIsTransientFor (latest_transient, c2))
+            {
+                latest_transient = c2;
+            }
+        }
     }
 
-    return latest_modal;
+    return latest_transient;
 }
 
 /* Build a GList of clients that have a transient relationship */
@@ -300,16 +331,16 @@ clientListTransient (Client * c)
     ScreenInfo *screen_info;
     Client *c2, *c3;
     GList *transients;
-    GList *index1, *index2;
+    GList *list1, *list2;
 
     g_return_val_if_fail (c != NULL, NULL);
 
     transients = NULL;
     screen_info = c->screen_info;
     transients = g_list_append (transients, c);
-    for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
+    for (list1 = screen_info->windows_stack; list1; list1 = g_list_next (list1))
     {
-        c2 = (Client *) index1->data;
+        c2 = (Client *) list1->data;
         if (c2 != c)
         {
             if (clientIsTransientFor (c2, c))
@@ -318,10 +349,10 @@ clientListTransient (Client * c)
             }
             else
             {
-                for (index2 = transients; index2;
-                    index2 = g_list_next (index2))
+                for (list2 = transients; list2;
+                    list2 = g_list_next (list2))
                 {
-                    c3 = (Client *) index2->data;
+                    c3 = (Client *) list2->data;
                     if ((c3 != c2) && clientIsTransientFor (c2, c3))
                     {
                         transients = g_list_append (transients, c2);
@@ -341,16 +372,16 @@ clientListTransientOrModal (Client * c)
     ScreenInfo *screen_info;
     Client *c2, *c3;
     GList *transients;
-    GList *index1, *index2;
+    GList *list1, *list2;
 
     g_return_val_if_fail (c != NULL, NULL);
 
     screen_info = c->screen_info;
     transients = NULL;
     transients = g_list_append (transients, c);
-    for (index1 = screen_info->windows_stack; index1; index1 = g_list_next (index1))
+    for (list1 = screen_info->windows_stack; list1; list1 = g_list_next (list1))
     {
-        c2 = (Client *) index1->data;
+        c2 = (Client *) list1->data;
         if (c2 != c)
         {
             if (clientIsTransientOrModalFor (c2, c))
@@ -359,10 +390,10 @@ clientListTransientOrModal (Client * c)
             }
             else
             {
-                for (index2 = transients; index2;
-                    index2 = g_list_next (index2))
+                for (list2 = transients; list2;
+                    list2 = g_list_next (list2))
                 {
-                    c3 = (Client *) index2->data;
+                    c3 = (Client *) list2->data;
                     if ((c3 != c2) && clientIsTransientOrModalFor (c2, c3))
                     {
                         transients = g_list_append (transients, c2);
@@ -382,15 +413,15 @@ gboolean
 clientCheckTransientWindow (Client *c, Window w)
 {
     GList *transients;
-    GList *index;
+    GList *list;
     Client *c2;
 
     g_return_val_if_fail (c != NULL, FALSE);
 
     transients = clientListTransient (c);
-    for (index = transients; index; index = g_list_next (index))
+    for (list = transients; list; list = g_list_next (list))
     {
-        c2 = (Client *) index->data;
+        c2 = (Client *) list->data;
         if (c2->window == w)
         {
             g_list_free (transients);
@@ -400,3 +431,18 @@ clientCheckTransientWindow (Client *c, Window w)
     g_list_free (transients);
     return TRUE;
 }
+
+gboolean
+clientSameApplication (Client *c1, Client *c2)
+{
+    g_return_val_if_fail (c1 != NULL, FALSE);
+    g_return_val_if_fail (c2 != NULL, FALSE);
+
+    return (clientIsTransientOrModalFor (c1, c2) ||
+            clientIsTransientOrModalFor (c2, c1) ||
+            clientSameGroup (c1, c2) ||
+            clientSameLeader (c1, c2) ||
+            (c1->pid != 0 && c1->pid == c2->pid &&
+             c1->hostname && c2->hostname &&
+             !g_strcasecmp (c1->hostname, c2->hostname)));
+}
